
// Cookie Defender

// General idea: 
// The system was built to provide a new way of communication between a parent and a child (generally 2 or more people).
// The communication is done through a cookie jar, and all its functionality is to provide different kinds of messages
// and cookie jar related data and functionality.

// What YOU should do when you use the system:
// 1. update your blynk app authentication code at line: 24.
// 2. update your wifi name and password at line: 25, 26.
// 3. ** Advanced: if you have problems with the values of the pins (which results a spam click on some of the buttons),
//    you should check the values and adjust them according to your system, at lines: 125, 129, 132.


#define BLYNK_PRINT SerialUSB
#define BLYNK_DEBUG SerialUSB

#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>
#include <Adafruit_CircuitPlayground.h>
#include <Time.h>

char auth[] = "";
char ssid[] = "";
char pass[] = "";

#define EspSerial Serial1
#define ESP8266_BAUD 115200

ESP8266 wifi(&EspSerial);

bool isClosed = true;
int failLvl = 0;
int failTimer = 0;
bool isFailedPw = false;
int successLvl = 0;
int successTimer = 0;
bool isSuccPw = false;
int oldCapTouchVal;
int CapTouchVal = analogRead(A5);
byte realPw[] =  {A1, A1, A1, A1};
byte pw[] =  {A5, A5, A5, A5};
int openIterCount = 0;
int pwIndex = 0;
byte pressIterCount[] = {
  25 , 25, 25, 25
};
byte capsensePins[] = {
  A1 , A2, A3, A4
};
int isPsd[] = { // is pressed? 0= not pressed , 1=pressed first time, 2=pressed not first time
  0, 0, 0, 0
};
float X, Y, Z, total, tempC;

int mood = 100; // 0 = Angry, 50 = Neutral, 100 = Happy

bool unauthOpen = false;
bool illeglAlready = false;
int lastMilis = 0;
int lastMilis2 = 0;
int lastMilis3 = 0;
int lastMilis4 = 0;

void setup() {
  CircuitPlayground.begin();
  SerialUSB.begin(9600);
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  Blynk.begin(auth, wifi, ssid, pass);
}

void loop() {

  if (!isFailedPw && !isSuccPw) {
    //////////////////////////////// Is the jar open?

    oldCapTouchVal = CapTouchVal;
    CapTouchVal = analogRead(A5);
    if (lastMilis4 == 0 || (millis() - lastMilis4) > 100) {

      lastMilis4 = millis();

    }
    if (CapTouchVal <= 40) {
      isClosed = true;
      openIterCount = 0;
    }
    if (CapTouchVal > 40) {
      openIterCount++;
      if (openIterCount == 15) {
        isClosed = false;
      }
    }
    //Serial.println(isClosed);
    ///////////////////////////////
    /////////////////////////////// Password

    for (int i = 0; i < 4; ++i) {
      int temp,psen = analogRead(capsensePins[i]);
      if (pressIterCount[i] == 0 && isPsd[i] == 1) {
        isPsd[i] = true;
        pw[pwIndex] = capsensePins[i];
        pwIndex = (pwIndex + 1) % 4;
        isPsd[i] = 2;
        if (pwIndex == 0) {
          int h = 0;
          for (h = 0; h < 4; ++h) {
            if (pw[h] != realPw[h]) {
              break;
            }
          }
          if (h == 4) {
            isSuccPw = true;
          } else {
            isFailedPw = true;
          }
        }
        break;
      }
      if (pressIterCount[i] >= 25) {
        isPsd[i] = 0;
      }
      if (isPsd[i] == 0 && (tempCpsen > 450 || tempCpsen <= 200)) {
        pressIterCount[i] = 0;
        isPsd[i] = 1;
      }
      if (isPsd[i] != 0 && (tempCpsen > 450 || tempCpsen <= 200)) {
        pressIterCount[i] = 0;
      }
      if (tempCpsen < 450 && tempCpsen >= 200) {
        pressIterCount[i]++;
      }
      
    }
   
    ///////////////////////////////
    /////////////////////////////// Accelerometer + Temp = mood
    X = CircuitPlayground.motionX();
    Y = CircuitPlayground.motionY();
    Z = CircuitPlayground.motionZ();

    total = abs(X) + abs(Y) + abs(Z);
    tempC = CircuitPlayground.temperature();

    if (total > 16.0 || total < 7 || tempC > 30) {
      mood = 0;
    } else if ((total > 10 && total < 13) || tempC < 20) {
      mood = 100;
    } else {
      mood = 50;
    }

    if (lastMilis == 0 || (millis() - lastMilis) > 5000) {
      Blynk.virtualWrite(V5, mood);
      lastMilis = millis();
    }
    ////////////////////////////////
    //////////////////////////////// Unauthorized open
    if (!isClosed) {
      unauthOpen = true;
      if (!illeglAlready) {
        Blynk.virtualWrite(V3, "Illegal open");
        Blynk.virtualWrite(V4, "was detected!");
      }
      illeglAlready = true;
    }
  }
  ////////////////////////////////
  //////////////////////////////// fail password
  if (isFailedPw) {

    if (failLvl == 4 && millis() - failTimer > 2400) {
      for (int i = 0; i < 10; i++) {
        CircuitPlayground.setPixelColor(i, 0xFF0000);
      }
      failLvl = (failLvl + 1) % 6;
    }
    if (failLvl == 3 && millis() - failTimer > 2000) {
      for (int i = 0; i < 10; i++) {
        CircuitPlayground.setPixelColor(i, 0x000000);
      }
      failLvl = (failLvl + 1) % 6;
    }
    if (failLvl == 2 && millis() - failTimer > 1200) {
      for (int i = 0; i < 10; i++) {
        CircuitPlayground.setPixelColor(i, 0xFF0000);
      }
      failLvl = (failLvl + 1) % 6;
    }
    if (failLvl == 1 && millis() - failTimer > 800) {
      for (int i = 0; i < 10; i++) {
        CircuitPlayground.setPixelColor(i, 0x000000);
      }
      failLvl = (failLvl + 1) % 6;
    }
    if (failLvl == 1) {
      CircuitPlayground.playTone(220, 15);
    }
    if (failLvl == 0) {
      for (int i = 0; i < 10; i++) {
        CircuitPlayground.setPixelColor(i, 0xFF0000);
      }
      failTimer = millis();
      failLvl = (failLvl + 1) % 6;
    }
    if (failLvl == 5 && millis() - failTimer > 3200) {
      for (int i = 0; i < 10; i++) {
        CircuitPlayground.setPixelColor(i, 0x000000);
      }
      isFailedPw = false;
      failLvl = (failLvl + 1) % 6;
    }
  }
  ////////////////////////////////
  //////////////////////////////// success password
  if (isSuccPw) {
    if (successLvl == 2 && millis() - successTimer > (7500 + 50 * mood)) {
      for (int i = 0; i < 10; i++) {
        CircuitPlayground.setPixelColor(i, 0xFF0000); //add good sound
      }
      successLvl = (successLvl + 1) % 4;
      successTimer = millis();
    }
    if (successLvl == 1 && millis() - successTimer > (7500 + 50 * mood)) {
      for (int i = 0; i < 10; i++) {
        CircuitPlayground.setPixelColor(i, 0xFFFF00);
      }
      successLvl = (successLvl + 1) % 4;
      successTimer = millis();
    }
    if (successLvl == 1 && (millis() - successTimer) < 1500 ) {
      CircuitPlayground.playTone(261, 15);
    }
    if (successLvl == 1 && (millis() - successTimer) > 1500 && (millis() - successTimer) < 2500 ) {
      CircuitPlayground.playTone(523, 15);
    }
    if (successLvl == 0) {
      for (int i = 0; i < 10; i++) {
        CircuitPlayground.setPixelColor(i, 0x00FF00); //add goo sound
      }
      successTimer = millis();
      successLvl = (successLvl + 1) % 4;
    }
    if (successLvl == 3 && millis() - successTimer >  (7500 + 50 * mood)) {
      for (int i = 0; i < 10; i++) {
        CircuitPlayground.setPixelColor(i, 0x000000);
      }
      isSuccPw = false;
      successLvl = (successLvl + 1) % 4;
    }
  }
  ////////////////////////////////
  //////////////////////////////// Blynk
  if (lastMilis3 == 0 || (millis() - lastMilis3) > 100) {
    Blynk.run();
    lastMilis3 = millis();
  }
}

BLYNK_WRITE(V0) {
  Blynk.virtualWrite(V3, "No new");
  Blynk.virtualWrite(V4, "notifications");
  illeglAlready = false;
}


BLYNK_WRITE(V1) {
  if (param.asInt() == 1) {
    isSuccPw = true; //openedJar(mood);
  }
}

BLYNK_WRITE(V2) {
  String blynkPw = param.asStr();
  Serial.println("Blinykpw: ");
  Serial.println(blynkPw);
  if (blynkPw == "") {
    //send please enter pw
    Serial.println("PLEASE ENTER PW");
  } else {
    int bPw = blynkPw.toInt();
    Serial.println(bPw);
    if (bPw > 999) {
      byte tmpPw[] =  {A6, A6, A6, A6};
      int i = 0;
      while (i < 4) {
        int tmp = bPw % 10;
        if (tmp > 4 || tmp < 1) {
          //send wrong digits
          Serial.println("WRONG DIGITS");
          break;
        } else {
          tmpPw[i] = capsensePins[tmp - 1];
        }
        bPw = bPw / 10;
        i++;
      }
      if (i == 4) {
        for (int j = 0; j < 4; j++) {
          realPw[j] = tmpPw[j];
        }
        Serial.println("YEYYY");
      } else {
        Serial.println("Please enter right digits");
      }
    } else {
      //Send pw too short
      Serial.println("PW TOO SHORT");
    }
  }
}
